/*
  LittleBits light and ultrasone sensitive MIDI music controller
  Alex P July 2022
  The Littlebits W6 Arduino Leonardo module can send MIDI signals via USB
  Updated from Light Sensitive MIDI Controller (Arduino) on pastebin.com
  Littlebit modules:
  main module is the W6 Arduino Leonardo module
  P1 power on input D0 for power supply
  power snap + I13 light sensor on input A0 for tempo selection
  power snap + I13 light sensor on input A1 for instrument selection and vibrato ON/OFF
  W9 proto bit on output D1 for power supply of the HC-SR04 ultrasone sensor
  D10 and D11 connection on the W6 arduino board for trigger and echo of the HC-SR04 ultrasone sensor
  O3 rgb LED on output D5 for vibrato ON/OFF indication
*/

//**********************LIBRARIES**************************************
#include "MIDIUSB.h"
#include "NewPing.h"

// Hook up HC-SR04 with Trig to Arduino Pin 9, Echo to Arduino pin 10
#define TRIGGER_PIN 10
#define ECHO_PIN 11

// Maximum distance we want to ping for (in centimeters).
#define MAX_DISTANCE 25

// NewPing setup of pins and maximum distance.
NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);

//**********************VARIABLES**************************************
int Distance = 0;
int LightPin1 = A1;
int LightPin2 = A0;
int LightLevel1 = 0;
int MinLight1 = 0;
int MaxLight1 = 0;
int LightLevel2 = 0;
int MinLight2 = 0;
int MaxLight2 = 0;
int Instrument = 0;
int ToneMap;
int TempoMap;
int nn;
int VibratoLedPin = 5;
int Vibrato = 0;
int VibratoMem = 0;
int Select = 0;
int Step = 0;
int InstrArray[16] = {0, 9, 10, 55, 68, 77, 86, 98, 99, 100, 102, 104, 106, 107, 112, 114};

void setup() {

  pinMode(VibratoLedPin, OUTPUT);
  pinMode(TRIGGER_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
  Serial.begin(9600);

  //**********************INITIAL LIGHT LEVELS DETECTION***********
  LightLevel2 = analogRead(LightPin2);
  MinLight2 = LightLevel2 - 20;
  MaxLight2 = LightLevel2;

  LightLevel1 = analogRead(LightPin1);
  MinLight1 = LightLevel1 - 20;
  MaxLight1 = LightLevel1;
}

void loop() {

  //**********************TEMPO LEVEL SETTING****************************
  LightLevel2 = analogRead(LightPin2);
  if (MinLight2 > LightLevel2) MinLight2 = LightLevel2;
  if (MaxLight2 < LightLevel2) MaxLight2 = LightLevel2;
  TempoMap = map(LightLevel2, MinLight2, MaxLight2, 2, 500);

  //**********************TONE SETTING***********************************
  ToneMap = map(sonar.ping_cm(), 0, 25, 24, 96);
  
  //**********************VIBRATO ON/OFF SETTING*************************
  if (Step % 2 == 0)
  {
    controlChange(1, 1, 127);
    digitalWrite(VibratoLedPin, HIGH);
    delay(100);
  }

  if (Step % 2 != 0)
  {
    controlChange(1, 1, 0);
    digitalWrite(VibratoLedPin, LOW);
    delay(100);
  }

  //**********************INSTRUMENT SELECTION***************************

  LightLevel1 = analogRead(LightPin1);
  if (MinLight1 > LightLevel1) MinLight1 = LightLevel1;
  if (MaxLight1 < LightLevel1) MaxLight1 = LightLevel1;

  Select = LightLevel1;
  if (Select < 600)
  {
    Instrument = InstrArray[Step];
    Step = Step + 1;
    if (Step > 15 ) {
      Step = 0;
    }
    programChange(1, Instrument);
    MidiUSB.flush();
  }


  //**********************PLAYING NOTES**********************************

  if (ToneMap > 25) {
    noteOn(1, ToneMap, 127); noteOn(1, ToneMap + 2, 127); noteOn(1, ToneMap + 4, 127);
    delay(TempoMap);
    noteOff(1, ToneMap, 127); noteOff(1, ToneMap + 2, 127); noteOff(1, ToneMap + 4, 127);

    MidiUSB.flush();
  }

  //**********************DEBUGGING VIA MONITOR****************************
  //Uncomment for debugging:
  //Serial.print(ToneMap);
  //Serial.println();
  //Serial.print(Select);
  //Serial.println();
  //Serial.print(Step);
  //Serial.println();
}

//**********************MIDI SUBROUTINES*********************************
void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel - 1, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
}

void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel - 1, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
}

void controlChange(byte channel, byte control, byte value) {
  midiEventPacket_t event = {0x0B, 0xB0 | channel - 1, control, value};
  MidiUSB.sendMIDI(event);
}
void programChange(byte channel, byte patch) {
  midiEventPacket_t event = {0x0C, 0xC0 | channel - 1, patch};
  MidiUSB.sendMIDI(event);
}
